package main

import (
	"archive/zip"
	"fmt"
	"github.com/jessevdk/go-flags"
	"os"
	"path/filepath"
	"reflect"
	"strings"
)

//
//	peco/peco · GitHub
// https://github.com/peco/peco
//
type cmdOptions struct {
	OptHelp bool `short:"h" long:"help" description:"show this help message and exit"`
}

func showHelp() {
	// The ONLY reason we're not using go-flags' help option is
	// because I wanted to tweak the format just a bit... but
	// there wasn't an easy way to do so
	os.Stderr.WriteString(`zlist 1.0 of 19 September 2014, by KOYAMA Yoshiaki.
Yoshiaki's HomePage [http://kyoshiaki.sakura.ne.jp/osx/index.html]
KOYAMA Yoshiaki blog [http://kyoshiaki.hatenablog.com/]

Usage: zlist [options] file[.zip]

Options:
`)

	t := reflect.TypeOf(cmdOptions{})
	for i := 0; i < t.NumField(); i++ {
		tag := t.Field(i).Tag

		var o string
		if s := tag.Get("short"); s != "" {
			o = fmt.Sprintf("-%s, --%s", tag.Get("short"), tag.Get("long"))
		} else {
			o = fmt.Sprintf("--%s", tag.Get("long"))
		}

		fmt.Fprintf(
			os.Stderr,
			"  %-21s %s\n",
			o,
			tag.Get("description"),
		)
	}
}

type ZipFile struct {
	Name               string
	CompressedSize64   uint64
	UncompressedSize64 uint64
	File               *zip.File
}

func (zf ZipFile) String() string {
	return fmt.Sprintf("%6d (%6d) %s", zf.UncompressedSize64, zf.CompressedSize64, zf.Name)
}
func (zf ZipFile) PrintlnHeader() {
	fmt.Printf("%6s (%s) %s\n", "size", "compressed size", "filenames")
}

func main() {
	var st int

	defer func() { os.Exit(st) }()

	opts := &cmdOptions{}
	p := flags.NewParser(opts, flags.PrintErrors)
	args, err := p.Parse()
	if err != nil || len(args) == 0 || opts.OptHelp {
		showHelp()
		st = 1
		return
	}

	name := args[0]
	ext := filepath.Ext(name)
	ext = strings.ToLower(ext)
	if ext != ".zip" {
		name = name + ".zip"
	}

	r, err := zip.OpenReader(name)
	if err != nil {
		fmt.Fprintf(os.Stderr, "Can't open %s\n", name)
		st = 1
		return
	}
	defer r.Close()

	fmt.Printf("%s:\n", name)

	files := make([]ZipFile, len(r.File))

	for index, item := range r.File {
		files[index] = ZipFile{Name: item.Name, CompressedSize64: item.CompressedSize64, UncompressedSize64: item.UncompressedSize64, File: r.File[index]}
	}

	for index, item := range files {
		if index == 0 {
			item.PrintlnHeader()
		}
		fmt.Println(item)
	}
}
